@@ -9,6 +9,7 @@ from io import StringIO
 
 import pypeta
 from pypeta import Peta
+import numba
 
 
 class SampleIdError(RuntimeError):
@@ -47,6 +48,11 @@ class ListsUnEqualLengthError(RuntimeError):
         self.message = message
 
 
+class DatetimeFormatError(RuntimeError):
+    def __init__(self, message: str):
+        self.message = message
+
+
 class CDx_Data():
     """[summary]
     """
@@ -70,8 +76,11 @@ class CDx_Data():
 
         if not cli_df is None:
             self.cli = cli_df
+            self.cli=self._infer_datetime_columns()
         else:
             self._set_cli()
+
+        
         self.crosstab = self.get_crosstab()
 
     def from_PETA(self, token: str, json_str: str):
@@ -92,6 +101,7 @@ class CDx_Data():
         self.cnv = peta.fetch_cnv_data()
         self.sv = peta.fetch_sv_data()
 
+        self.cli=self._infer_datetime_columns()
         self.crosstab = self.get_crosstab()
 
     def from_file(self,
@@ -121,6 +131,7 @@ class CDx_Data():
         else:
             self._set_cli()
 
+        self.cli=self._infer_datetime_columns()
         self.crosstab = self.get_crosstab()
 
     def write_files(self, path: str = './'):
@@ -168,6 +179,23 @@ class CDx_Data():
             }).drop_duplicates()
         else:
             self.cli = None
+        
+    def _infer_datetime_columns(self)->pd.DataFrame:
+        """To infer the datetime_columns and astype to datetime64 format
+
+        Returns:
+            pd.DataFrame: CDx.cli dataframe
+        """
+        cli=self.cli
+        for column in cli.columns:            
+            if column.endswith('DATE'):
+                try:
+                    cli[column]=pd.to_datetime(cli[column])
+                except Exception as e:
+                    raise DatetimeFormatError(f'{column} column end with "DATE" can not be transformed to datetime format')
+
+        return cli
+
 
     def get_crosstab(self) -> pd.DataFrame:
         """Generate a Gene vs. Sample_id cross table.
@@ -179,7 +207,7 @@ class CDx_Data():
             pd.DataFrame: CDx_Data.
         """
         # 这里cli表中不允许存在相同的样本编号。会造成crosstab的列中存在重复，引入Series的boolen值无法处理的问题
-        if self.cli is None:
+        if (self.cli is None) or (len(self.cli) == 0):
             return pd.DataFrame([])
 
         sub_dfs = []
@@ -189,9 +217,10 @@ class CDx_Data():
         sub_dfs.append(cli_crosstab)
 
         # mut. represent by cHgvs, joined by '|' for mulitple hit
-        if not self.mut is None:
-            mut_undup = self.mut.drop_duplicates(
-                subset=['Hugo_Symbol', 'Tumor_Sample_Barcode'])
+        if (not self.mut is None) and (len(self.mut) != 0):
+            mut_undup = self.mut[['Hugo_Symbol', 'Tumor_Sample_Barcode', 'HGVSp_Short'
+                    ]].groupby(['Hugo_Symbol', 'Tumor_Sample_Barcode'
+                        ])['HGVSp_Short'].apply(lambda x: '|'.join(x)).reset_index()
             mut_crosstab = mut_undup.pivot('Hugo_Symbol',
                                            'Tumor_Sample_Barcode',
                                            'HGVSp_Short')
@@ -199,10 +228,11 @@ class CDx_Data():
 
             sub_dfs.append(mut_crosstab)
 
-        # cnv. represent by gain or loss. at first use the virtual column "status"
-        if not self.cnv is None:
-            cnv_undup = self.cnv.drop_duplicates(
-                subset=['Hugo_Symbol', 'Tumor_Sample_Barcode'])
+        # cnv. represent by gain or loss. at first use the virtual column "copy_Num"
+        if (not self.cnv is None) and (len(self.cnv) != 0):
+            cnv_undup = self.cnv[['Hugo_Symbol','Tumor_Sample_Barcode', 'status'
+                ]].groupby(['Hugo_Symbol', 'Tumor_Sample_Barcode'
+                    ])['status'].apply(lambda x: '|'.join(x)).reset_index()
             cnv_crosstab = cnv_undup.pivot('Hugo_Symbol',
                                            'Tumor_Sample_Barcode', 'status')
             cnv_crosstab['track_type'] = 'CNV'
@@ -210,14 +240,16 @@ class CDx_Data():
             sub_dfs.append(cnv_crosstab)
 
         # sv. represent by gene1 and gene2 combination. explode one record into 2 lines.
-        if not self.sv is None:
+        if (not self.sv is None) and (len(self.sv) != 0):
             sv_undup = pd.concat([
                 self.sv,
                 self.sv.rename(columns={
                     'gene1': 'gene2',
                     'gene2': 'gene1'
                 })
-            ]).drop_duplicates(subset=['gene1', 'Tumor_Sample_Barcode'])
+            ])[['gene1', 'Tumor_Sample_Barcode', 'gene2'
+                ]].groupby(['gene1', 'Tumor_Sample_Barcode'
+                    ])['gene2'].apply(lambda x: '|'.join(x)).reset_index()
             sv_crosstab = sv_undup.pivot('gene1', 'Tumor_Sample_Barcode',
                                          'gene2')
             sv_crosstab['track_type'] = 'FUSION'
@@ -493,6 +525,31 @@ class CDx_Data():
         cli_df = self._selector(self.cli, kwargs)
         return self.select_by_sample_ids(cli_df['sampleId'])
 
+
+    def select_samples_by_date_attributes(self,column_name:str='SAMPLE_RECEIVED_DATE',
+        start='',end:str='',
+        days:int=0,period:str='',):
+        """Select samples using a datetime attribute in the cli dataframe
+
+        Args:
+            column_name (str, optional): Column used in the cli dataframe. Defaults to 'SAMPLE_RECEIVED_DATE'.
+            from (str, optional): Time start point. Defaults to ''.
+            to (str, optional): Time end point. Defaults to ''.
+            days (int, optional): Days lasts. Defaults to ''.
+            exact (str, optional): Exact range,eg '202005' for May in 2020 or '2021' for the whole year. Defaults to ''.
+        """
+        date_ser=self.cli.set_index(column_name)['sampleId']
+        if period:
+            cdx=self.select_by_sample_ids(date_ser[period])
+        elif start and end:
+            cdx=self.select_by_sample_ids(date_ser[start:end])
+        elif start and days:
+            cdx=self.select_by_sample_ids(date_ser[start:(pd.to_datetime(start)+pd.to_timedelta(days,'D')).strftime("%Y-%m-%d")])
+        elif end and days:
+            cdx=self.select_by_sample_ids(date_ser[(pd.to_datetime(end) - pd.to_timedelta(days,'D')).strftime("%Y-%m-%d"):end])
+        
+        return cdx
+
     # 对阳性样本进行选取。基因组合，且或关系，chgvs和ghgvs，基因系列如MMR、HR等
     # 基因组合可以做为入参数组来传入
     def select_samples_by_mutate_genes(
@@ -685,16 +742,17 @@ class CDx_Data():
                 self.crosstab.loc['CLINICAL', groupby],
                 axis=1).apply(self._crosstab_to_positive_rate)
         elif groupby_genes:
-            test_posi_rate = crosstab.groupby(level=1).apply(
+            test_posi_rate = crosstab.groupby(level=1,sort=False).apply(
                 self._crosstab_to_positive_rate)
         elif groupby_variant_type:
-            test_posi_rate = crosstab.groupby(level=0).apply(
+            test_posi_rate = crosstab.groupby(level=0,sort=False).apply(
                 self._crosstab_to_positive_rate)
         else:
             test_posi_rate = self._crosstab_to_positive_rate(crosstab)
 
         return test_posi_rate
 
+
     def _crosstab_to_positive_rate(self, df: pd.DataFrame):
         """Calculate a crosstab to generate a positive rate value for notnull cell
 
@@ -708,6 +766,7 @@ class CDx_Data():
                                         [True])[-1]
         return posi_rate
 
+
     def _positive_rate(self, values: list,
                        positive_tags: list) -> Tuple[int, int, float]:
         """Calculate positive tags marked values percentage in the total ones
